@@ -8,7 +8,6 @@ import subprocess
 import sys
 import urllib.error
 from datetime import datetime, timezone
-from pathlib import Path
 from textwrap import dedent
 from typing import get_args
 from urllib.parse import urlparse
@@ -40,6 +39,7 @@ from ramalama.config import (
     load_file_config,
 )
 from ramalama.endian import EndianMismatchError
+from ramalama.log_levels import LogLevel
 from ramalama.logger import configure_logger, logger
 from ramalama.model_inspect.error import ParseError
 from ramalama.model_store.global_store import GlobalModelStore
@@ -60,6 +60,8 @@ from ramalama.version import print_version, version
 shortnames = Shortnames()
 
 GENERATE_OPTIONS = ["quadlet", "kube", "quadlet/kube", "compose"]
+LIST_SORT_FIELD_OPTIONS = ["size", "modified", "name"]
+LIST_SORT_ORDER_OPTIONS = ["desc", "asc"]
 
 
 class ParsedGenerateInput:
@@ -80,8 +82,8 @@ class ParsedGenerateInput:
 def parse_generate_option(option: str) -> ParsedGenerateInput:
     # default output directory is where ramalama has been started from
     generate, output_dir = option, "."
-    if generate.count(":") == 1:
-        generate, output_dir = generate.split(":")
+    if generate.count(":") > 0:
+        generate, output_dir = generate.split(":", 1)
     if output_dir == "":
         output_dir = "."
 
@@ -133,8 +135,6 @@ def available_metadata(prefix, parsed_args, **kwargs):
         # shotnames resolution has not been applied for auto-completion
         # Therefore it needs to be done explicitly here in order to support it
         resolved_model = shortnames.resolve(parsed_args.MODEL)
-        if not resolved_model:
-            resolved_model = parsed_args.MODEL
 
         metadata = New(resolved_model, parsed_args).inspect_metadata()
         return [field for field in metadata.keys() if field.startswith(parsed_args.get)]
@@ -313,12 +313,38 @@ def parse_arguments(parser):
 
 def post_parse_setup(args):
     """Perform additional setup after parsing arguments."""
+
+    def map_https_to_transport(input: str) -> str | None:
+        if input.startswith("https://") or input.startswith("http://"):
+            url = urlparse(input)
+            # detect if the whole repo is defined or a specific file
+            if url.path.count("/") != 2:
+                return input
+            if url.hostname in ["hf.co", "huggingface.co"]:
+                return f"hf:/{url.path}"
+            if url.hostname in ["ollama.com"]:
+                return f"ollama:/{url.path}"
+        return input
+
+    # Resolve the model input
+    # First, map https:// inputs to ollama or huggingface based on url domain
+    # Then resolve the input based on the available shortname list
     if getattr(args, "MODEL", None):
-        if args.subcommand != "rm":
-            resolved_model = shortnames.resolve(args.MODEL)
-            if resolved_model:
-                args.UNRESOLVED_MODEL = args.MODEL
-                args.MODEL = resolved_model
+        if isinstance(args.MODEL, str):
+            args.INITIAL_MODEL = args.MODEL
+            args.MODEL = map_https_to_transport(args.MODEL)
+
+            args.UNRESOLVED_MODEL = args.MODEL
+            args.MODEL = shortnames.resolve(args.MODEL)
+
+        if isinstance(args.MODEL, list):
+            args.INITIAL_MODEL = [m for m in args.MODEL]
+            for i in range(len(args.MODEL)):
+                args.MODEL[i] = map_https_to_transport(args.MODEL[i])
+
+            args.UNRESOLVED_MODEL = [m for m in args.MODEL]
+            for i in range(len(args.MODEL)):
+                args.MODEL[i] = shortnames.resolve(args.MODEL[i])
 
         # TODO: This is a hack. Once we have more typing in place these special cases
         # _should_ be removed.
@@ -346,7 +372,8 @@ def post_parse_setup(args):
     if hasattr(args, 'pull'):
         args.pull = normalize_pull_arg(args.pull, getattr(args, 'engine', None))
 
-    configure_logger("DEBUG" if args.debug else "WARNING")
+    log_level = LogLevel.DEBUG if args.debug else (CONFIG.log_level or LogLevel.WARNING)
+    configure_logger(log_level)
 
 
 def login_parser(subparsers):
@@ -435,24 +462,6 @@ def human_duration(d):
     return "1 year" if d < 63072000 else f"{d // 31536000} years"
 
 
-def list_files_by_modification(args):
-    paths = Path().rglob("*")
-    models = []
-    for path in paths:
-        if str(path).startswith("file/"):
-            if not os.path.exists(str(path)):
-                path = str(path).replace("file/", "file:///")
-                perror(f"{path} does not exist")
-                continue
-        if os.path.exists(path):
-            models.append(path)
-        else:
-            perror(f"Broken symlink found in: {args.store}/models/{path} \nAttempting removal")
-            New(str(path).replace("/", "://", 1), args).remove(args)
-
-    return sorted(models, key=lambda p: os.path.getmtime(p), reverse=True)
-
-
 def bench_cli(args):
     model = New(args.MODEL, args)
     model.ensure_model_exists(args)
@@ -517,6 +526,20 @@ def list_parser(subparsers):
     parser.add_argument("--all", dest="all", action="store_true", help="include partially downloaded AI Models")
     parser.add_argument("--json", dest="json", action="store_true", help="print using json")
     parser.add_argument("-n", "--noheading", dest="noheading", action="store_true", help="do not display heading")
+    parser.add_argument(
+        "--sort",
+        dest="sort",
+        choices=LIST_SORT_FIELD_OPTIONS,
+        default="name",
+        help="field used to sort the AI Models",
+    )
+    parser.add_argument(
+        "--order",
+        dest="order",
+        choices=LIST_SORT_ORDER_OPTIONS,
+        default="desc",
+        help="order used to sort the AI Models",
+    )
     parser.set_defaults(func=list_cli)
 
 
@@ -531,24 +554,13 @@ def human_readable_size(size):
     return f"{size} PB"
 
 
-def get_size(path):
-    if os.path.isdir(path):
-        size = 0
-        for dirpath, _, filenames in os.walk(path, followlinks=True):
-            for file in filenames:
-                filepath = os.path.join(dirpath, file)
-                if not os.path.islink(filepath):
-                    size += os.path.getsize(filepath)
-        return size
-    return os.path.getsize(path)
-
-
 def _list_models_from_store(args):
     models = GlobalModelStore(args.store).list_models(engine=args.engine, show_container=args.container)
 
     ret = []
     local_timezone = datetime.now().astimezone().tzinfo
 
+    # map the listed models to the proper output structure for display
     for model, files in models.items():
         is_partially_downloaded = any(file.is_partial for file in files)
         if not args.all and is_partially_downloaded:
@@ -569,6 +581,9 @@ def _list_models_from_store(args):
             }
         )
 
+    # sort the listed models according to the desired order
+    ret.sort(key=lambda entry: entry[args.sort], reverse=args.order == "desc")
+
     return ret
 
 
@@ -719,11 +734,12 @@ def convert_parser(subparsers):
     )
     parser.add_argument(
         "--type",
-        default="raw",
-        choices=["car", "raw"],
+        default=CONFIG.convert_type,
+        choices=["artifact", "car", "raw"],
         help="""\
 type of OCI Model Image to push.
 
+Model "artifact" stores the AI Model as an OCI Artifact.
 Model "car" includes base image with the model stored in a /models subdir.
 Model "raw" contains the model and a link file model.file to it stored at /.""",
     )
@@ -738,8 +754,6 @@ def convert_cli(args):
 
     target = args.TARGET
     tgt = shortnames.resolve(target)
-    if not tgt:
-        tgt = target
 
     model = TransportFactory(tgt, args).create_oci()
 
@@ -762,11 +776,12 @@ def push_parser(subparsers):
     add_network_argument(parser)
     parser.add_argument(
         "--type",
-        default="raw",
-        choices=["car", "raw"],
+        default=CONFIG.convert_type,
+        choices=["artifact", "car", "raw"],
         help="""\
 type of OCI Model Image to push.
 
+Model "artifact" stores the AI Model as an OCI Artifact.
 Model "car" includes base image with the model stored in a /models subdir.
 Model "raw" contains the model and a link file model.file to it stored at /.""",
     )
@@ -781,12 +796,12 @@ Model "raw" contains the model and a link file model.file to it stored at /.""",
     parser.set_defaults(func=push_cli)
 
 
-def _get_source_model(args):
+def _get_source_model(args, transport=None):
     src = shortnames.resolve(args.SOURCE)
-    if not src:
-        src = args.SOURCE
-    smodel = New(src, args)
+    smodel = New(src, args, transport=transport)
     if smodel.type == "OCI":
+        if not args.TARGET:
+            return smodel
         raise ValueError(f"converting from an OCI based image {src} is not supported")
     if not smodel.exists() and not args.dryrun:
         smodel.pull(args)
@@ -794,12 +809,14 @@ def _get_source_model(args):
 
 
 def push_cli(args):
-    source_model = _get_source_model(args)
     target = args.SOURCE
+    transport = None
+    if not args.TARGET:
+        transport = "oci"
+    source_model = _get_source_model(args, transport=transport)
+
     if args.TARGET:
         target = shortnames.resolve(args.TARGET)
-        if not target:
-            target = args.TARGET
     target_model = New(target, args)
 
     try:
@@ -1057,6 +1074,13 @@ def chat_run_options(parser):
     )
     parser.add_argument("--prefix", type=str, help="prefix for the user prompt", default=default_prefix())
     parser.add_argument("--mcp", nargs="*", help="MCP servers to use for the chat")
+    parser.add_argument(
+        "--summarize-after",
+        type=int,
+        default=CONFIG.summarize_after,
+        metavar="N",
+        help="automatically summarize conversation history after N messages to prevent context growth (0=disabled)",
+    )
 
 
 def chat_parser(subparsers):
@@ -1173,9 +1197,14 @@ def serve_cli(args):
         model.ensure_model_exists(args)
     except KeyError as e:
         try:
+            if "://" in args.MODEL:
+                raise e
             args.quiet = True
             model = TransportFactory(args.MODEL, args, ignore_stderr=True).create_oci()
             model.ensure_model_exists(args)
+            # Since this is a OCI model, prepend oci://
+            args.MODEL = f"oci://{args.MODEL}"
+
         except Exception:
             raise e
 
@@ -1416,29 +1445,42 @@ def rm_parser(subparsers):
     parser.set_defaults(func=rm_cli)
 
 
+def _rm_oci_model(model, args) -> bool:
+    # attempt to remove as a container image
+    try:
+        m = TransportFactory(model, args, ignore_stderr=True).create_oci()
+        return m.remove(args)
+    except Exception:
+        return False
+
+
 def _rm_model(models, args):
+    exceptions = []
     for model in models:
-        resolved_model = shortnames.resolve(model)
-        if resolved_model:
-            model = resolved_model
+        model = shortnames.resolve(model)
 
         try:
             m = New(model, args)
-            m.remove(args)
-        except KeyError as e:
+            if m.remove(args):
+                continue
+            # Failed to remove and might be OCI so attempt to remove OCI
+            if args.ignore:
+                _rm_oci_model(model, args)
+                continue
+        except (KeyError, subprocess.CalledProcessError) as e:
             for prefix in MODEL_TYPES:
                 if model.startswith(prefix + "://"):
                     if not args.ignore:
                         raise e
-            try:
-                # attempt to remove as a container image
-                m = TransportFactory(model, args, ignore_stderr=True).create_oci()
-                m.remove(args)
-                return
-            except Exception:
-                pass
-            if not args.ignore:
-                raise e
+            # attempt to remove as a container image
+            if _rm_oci_model(model, args) or args.ignore:
+                continue
+            exceptions.append(e)
+
+    if len(exceptions) > 0:
+        for exception in exceptions[1:]:
+            perror("Error: " + str(exception).strip("'\""))
+        raise exceptions[0]
 
 
 def rm_cli(args):
@@ -1498,11 +1540,16 @@ def inspect_parser(subparsers):
 def inspect_cli(args):
     args.pull = "never"
     model = New(args.MODEL, args)
-    model.inspect(args.all, args.get == "all", args.get, args.json, args.dryrun)
+    print(model.inspect(args.all, args.get == "all", args.get, args.json, args.dryrun))
 
 
 def main() -> None:
     def eprint(e, exit_code):
+        try:
+            if args.debug:
+                logger.exception(e)
+        except Exception:
+            pass
         perror("Error: " + str(e).strip("'\""))
         sys.exit(exit_code)
 
@@ -1525,12 +1572,16 @@ def main() -> None:
         args.func(args)
     except urllib.error.HTTPError as e:
         eprint(f"pulling {e.geturl()} failed: {e}", errno.EINVAL)
+    except FileNotFoundError as e:
+        eprint(e, errno.ENOENT)
     except HelpException:
         parser.print_help()
-    except (ConnectionError, IndexError, KeyError, ValueError, NoRefFileFound) as e:
+    except (IsADirectoryError, ConnectionError, IndexError, KeyError, ValueError, NoRefFileFound) as e:
         eprint(e, errno.EINVAL)
     except NotImplementedError as e:
         eprint(e, errno.ENOSYS)
+    except subprocess.TimeoutExpired as e:
+        eprint(e, errno.ETIMEDOUT)
     except subprocess.CalledProcessError as e:
         eprint(e, e.returncode)
     except EndianMismatchError:
