@@ -1,11 +1,11 @@
 import json
 import os
 from dataclasses import dataclass
-from enum import StrEnum
 from pathlib import Path
 from typing import Optional
 
 from ramalama.common import generate_sha256, sanitize_filename
+from ramalama.compat import StrEnum
 from ramalama.logger import logger
 
 
